/*
 * coords.h:
 *
 * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 *
 * $Id:$
 *
 */

#ifndef __COORDS_H_ /* include guard */
#define __COORDS_H_

/* struct datum:
 * Definition of a datum (name, ellipsoid parameters).
 */
struct datum {
    char *name;
    short ellipsoid;
    short dx;
    short dy;
    short dz;
};

/* struct ellipsoid:
 * Definition of an ellipsoid.
 */
struct ellipsoid {
    char *name;     /* name of ellipsoid */
    double a;       /* semi-major axis, meters */
    double invf;    /* 1/f */
};

/* Datums themselves are defined in datum.c. */

#define DATUM_WGS84     100
#define DATUM_IRISH     39
#define DATUM_OSGB      75

/* grid.c */
int geog_to_osgb(double lat, double lon, char *zone, double *x, double *y, int datum);
int osgb_to_geog(char *zone, double x, double y, double *lat, double *lon, int datum);
int geog_to_irish(double lat, double lon, char *zone, double *x, double *y, int datum);
int irish_to_geog(char *zone, double x, double y, double *lat, double *lon, int datum);

/* latlong.c */
void geog_to_wgs84(double *lat, double *lon, int datum);
void wgs84_to_geog(double *lat, double *lon, int datum);

#endif /* __COORDS_H_ */
