/*
 * main.c:
 * Simple conversion program for coordinates.
 *
 * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 *
 */

static const char rcsid[] = "$Id:$";

#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "coords.h"

/* main:
 * Entry point. Should be called passing a decimal latitude and longitude.
 */
int main(int argc, char *argv[]) {
    char *p;
    double lat, lon, lat1, lon1;
    char zone[3];
    double east, north;
 
    errno = 0;
    lat = strtod(argv[1], &p);
    if (p == argv[1] || errno == ERANGE || lat > 90. || lat < -90.) {
        fprintf(stderr, "%s is not a valid latitude\n", argv[1]);
        return -1;
    }

    errno = 0;
    lon = strtod(argv[2], &p);
    if (p == argv[2] || errno == ERANGE || lon > 180. || lon < -180.) {
        fprintf(stderr, "%s is not a valid longitude\n", argv[2]);
        return -1;
    }

    printf("WGS84 lat/lon:         %12lf %12lf\n", lat, lon);

    /* Convert to the Airy spheroid. */
    lat1 = lat;
    lon1 = lon;
    wgs84_to_geog(&lat1, &lon1, DATUM_OSGB);
    printf("Airy lat/lon:          %12lf %12lf\n", lat1, lon1);

    /* Obtain OS grid reference. */
    if (!geog_to_osgb(lat1, lon1, zone, &east, &north, DATUM_WGS84))
        printf("OSGB grid ref:            n/a\n");
    else
        printf("OSGB grid ref:            %s %05d %05d\n", zone, (int)east, (int)north);

    /* Convert to the Modified Airy spheroid. */
    lat1 = lat;
    lon1 = lon;
    wgs84_to_geog(&lat1, &lon1, DATUM_OSGB);
    printf("Modified Airy lat/lon: %12lf %12lf\n", lat1, lon1);

    if (!geog_to_irish(lat1, lon1, zone, &east, &north, DATUM_WGS84))
        printf("Irish grid ref:           n/a\n");
    else
        printf("Irish grid ref:           %s %05d %05d\n", zone, (int)east, (int)north);
}
