#include "project.h"


/* Convert landranger grid of form XX NNNNN NNNNN into map data*/


static letter_to_x(char c)
{
  switch (c | 32) {
  case 'a':
  case 'f':
  case 'l':
  case 'q':
  case 'v':
    return 0;
  case 'b':
  case 'g':
  case 'm':
  case 'r':
  case 'w':
    return 1;
  case 'c':
  case 'h':
  case 'n':
  case 's':
  case 'x':
    return 2;
  case 'd':
  case 'j':
  case 'o':
  case 't':
  case 'y':
    return 3;
  case 'e':
  case 'k':
  case 'p':
  case 'u':
  case 'z':
    return 4;
  }
}


static letter_to_y(char c)
{
  switch (c | 32) {
  case 'a':
  case 'b':
  case 'c':
  case 'd':
  case 'e':
    return 4;
  case 'f':
  case 'g':
  case 'h':
  case 'j':
  case 'k':
    return 3;
  case 'l':
  case 'm':
  case 'n':
  case 'o':
  case 'p':
    return 2;
  case 'q':
  case 'r':
  case 's':
  case 't':
  case 'u':
    return 1;
  case 'v':
  case 'w':
  case 'x':
  case 'y':
  case 'z':
    return 0;
  }
}

static char xy_to_letter(int x, int y)
{
  char tbl[5][5] = { {'V', 'W', 'X', 'Y', 'Z'},
  {'Q', 'R', 'S', 'T', 'U'},
  {'L', 'M', 'N', 'O', 'P'},
  {'F', 'G', 'H', 'J', 'K'},
  {'A', 'B', 'C', 'D', 'E'}
  };

  if (x < 0)
    return ' ';
  if (y < 0)
    return ' ';
  if (x > 4)
    return ' ';
  if (y > 4)
    return ' ';

  return tbl[y][x];
}

print_bosxy(BOSXY o)
{
  printf("%ld %ld\n", o.x, o.y);
}

print_bosgrid(BOSGRID g)
{
  printf("%c%c%05d%05d\n", g.big, g.med, g.e, g.n);
}

sprint_bosgrid(char *buf, BOSGRID g)
{
  sprintf(buf, "%c%c %05d %05d", g.big, g.med, g.e, g.n);
}

print_bosmap(BOSMAP m)
{
  printf("%c%c%d%d%c%c%d%d %f %f\n",
         m.big, m.med, m.be, m.bn, m.mn, m.me, m.se, m.sn, m.x, m.y);
}


BOSMAP bosgrid_to_bosmap(BOSGRID g)
{
  BOSMAP m;
  int e = g.e;
  int n = g.n;

  m.big = g.big;
  m.med = g.med;

  m.be = e / 10000;
  m.bn = n / 10000;

  e -= m.be * 10000;
  n -= m.bn * 10000;

  if (n < 5000) {
    m.mn = 'S';
  } else {
    m.mn = 'N';
    n -= 5000;
  }

  if (e < 5000) {
    m.me = 'W';
  } else {
    m.me = 'E';
    e -= 5000;
  }


  m.se = e / 1000;
  m.sn = n / 1000;


  e -= (1000 * m.se);
  n -= (1000 * m.sn);

  m.se *= 2;
  m.sn *= 2;


  m.x = ((float) e) / 1000.f;
  m.y = ((float) n) / 1000.f;

  return m;
}

BOSGRID bosmap_to_bosgrid(BOSMAP m)
{
  BOSGRID g;
  int e = (int) ((1000.f * m.x) + 0.5);
  int n = (int) ((1000.f * m.y) + 0.5);

  m.se = m.se / 2;
  m.sn = m.sn / 2;

  e += 1000 * m.se;
  n += 1000 * m.sn;

  if (m.mn == 'N')
    n += 5000;
  if (m.me == 'E')
    e += 5000;

  e += m.be * 10000;
  n += m.bn * 10000;

  g.big = m.big;
  g.med = m.med;
  g.e = e;
  g.n = n;


  return g;
}


BOSXY bosgrid_to_bosxy(BOSGRID g)
{
  BOSXY o;

  o.x = g.e;
  o.y = g.n;

  o.x += 100000 * (letter_to_x(g.med) + (5 * letter_to_x(g.big)));
  o.y += 100000 * (letter_to_y(g.med) + (5 * letter_to_y(g.big)));

  o.x -= 1000000;
  o.y -= 500000;


  return o;
}


BOSGRID bosxy_to_bosgrid(BOSXY o)
{
  int x, y;
  BOSGRID g;

  o.x += 1000000;
  o.y += 500000;

  x = o.x / 500000;
  y = o.y / 500000;

  g.big = xy_to_letter(x, y);

  o.x -= x * 500000;
  o.y -= y * 500000;

  x = o.x / 100000;
  y = o.y / 100000;

  g.med = xy_to_letter(x, y);

  o.x -= x * 100000;
  o.y -= y * 100000;

  g.e = o.x;
  g.n = o.y;

  return g;
}

BOSXY bosxy_neighbour(BOSXY o, int n, int e)
{
  o.x += 1000L * (long) n;
  o.y += 1000L * (long) e;
  return o;
}

char *bosmap_to_filename(BOSMAP m)
{
  char buf[1024];
  sprintf(buf, "%s/1km/%c%c/%d%d/%c%c%d%d%c%c%d%d.gif", getenv("BOSPATH"),
          m.big, m.med, m.be, m.bn, m.big, m.med, m.be, m.bn,
          m.mn, m.me, m.se, m.sn);

  return strdup(buf);

}

int bos_same_map(BOSMAP m1, BOSMAP m2)
{
  if (m1.big != m2.big)
    return 0;
  if (m1.med != m2.med)
    return 0;
  if (m1.be != m2.be)
    return 0;
  if (m1.bn != m2.bn)
    return 0;
  if (m1.mn != m2.mn)
    return 0;
  if (m1.me != m2.me)
    return 0;
  if (m1.se != m2.se)
    return 0;
  if (m1.sn != m2.sn)
    return 0;
  return 1;
}
